@@ -1,37 +1,23 @@
-import logging
-
 from django.conf import settings
 from django.shortcuts import get_object_or_404
 from rest_framework import filters, viewsets, mixins
 from rest_framework.decorators import action
-from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated
+from rest_framework.permissions import DjangoModelPermissions
 from rest_framework.response import Response
 from rest_framework.status import HTTP_204_NO_CONTENT
+from reversion import set_comment
+from reversion.views import RevisionMixin
 
 from huscy.subjects import pagination, models, serializers, services
-from huscy.subjects.permissions import (
-    ChangeSubjectPermission,
-    NotePermission,
-    ViewSubjectPermission,
-)
-
-logger = logging.getLogger('huscy.subjects')
+from huscy.subjects.permissions import ChangeSubjectPermission, ViewSubjectPermission
 
 
-def get_client_ip(request):
-    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
-    if x_forwarded_for:
-        ip = x_forwarded_for.split(',')[0]
-    else:
-        ip = request.META.get('REMOTE_ADDR')
-    return ip
-
-
-class SubjectViewSet(viewsets.ModelViewSet):
+class SubjectViewSet(RevisionMixin, viewsets.ModelViewSet):
     filter_backends = (
         filters.OrderingFilter,
         filters.SearchFilter,
     )
+    http_method_names = 'get', 'post', 'put', 'delete', 'head', 'options', 'trace'
     ordering_fields = (
         'contact__date_of_birth',
         'contact__first_name',
@@ -51,9 +37,16 @@ class SubjectViewSet(viewsets.ModelViewSet):
         )
 
     def perform_create(self, serializer):
-        serializer.save()
-        logger.info('User %s (IP: %s) created new subject',
-                    self.request.user, get_client_ip(self.request))
+        subject = serializer.save()
+        set_comment(f'Created subject <ID-{subject.id}>')
+
+    def perform_destroy(self, subject):
+        services.delete_subject(subject)
+        set_comment(f'Deleted subject <ID-{subject.id}')
+
+    def perform_update(self, serializer):
+        subject = serializer.save()
+        set_comment(f'Updated subject <ID-{subject.id}>')
 
     def list(self, request):
         '''
@@ -76,46 +69,38 @@ class SubjectViewSet(viewsets.ModelViewSet):
 
     def _delete_inactivity(self):
         services.unset_inactivity(self.get_object())
+        set_comment('Unset inactivity')
         return Response(status=HTTP_204_NO_CONTENT)
 
     def _create_inactivity(self, request):
         serializer = serializers.InactivitySerializer(data=request.data)
         serializer.is_valid(raise_exception=True)
-        serializer.save()
+        inactivity = serializer.save()
+        set_comment(f'Set inactivity: {inactivity}')
         return Response(serializer.data)
 
 
-class GuardianViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.UpdateModelMixin,
-                      viewsets.GenericViewSet):
+class GuardianViewSet(RevisionMixin, mixins.CreateModelMixin, mixins.DestroyModelMixin,
+                      mixins.UpdateModelMixin, viewsets.GenericViewSet):
+    http_method_names = 'post', 'put', 'delete', 'head', 'options', 'trace'
     permission_classes = (ChangeSubjectPermission, )
-    queryset = models.Contact.objects.all()
     serializer_class = serializers.GuardianSerializer
 
     def initial(self, request, *args, **kwargs):
         super().initial(request, *args, **kwargs)
         self.subject = get_object_or_404(models.Subject, pk=self.kwargs['subject_pk'])
 
-    def get_serializer_context(self):
-        context = super().get_serializer_context()
-        context['subject'] = self.subject
-        return context
+    def get_queryset(self):
+        return self.subject.guardians.all()
+
+    def perform_create(self, serializer):
+        guardian = serializer.save(subject=self.subject)
+        set_comment(f'Created guardian <ID-{guardian.id}>')
 
     def perform_destroy(self, guardian):
         services.remove_guardian(self.subject, guardian)
+        set_comment(f'Deleted guardian <ID-{guardian.id}>')
 
-
-class NoteViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, viewsets.GenericViewSet):
-    permission_classes = (IsAuthenticated, NotePermission)
-    serializer_class = serializers.NoteSerializer
-
-    def initial(self, request, *args, **kwargs):
-        super().initial(request, *args, **kwargs)
-        self.subject = get_object_or_404(models.Subject, pk=self.kwargs['subject_pk'])
-
-    def get_queryset(self):
-        return services.get_notes(self.subject)
-
-    def get_serializer_context(self):
-        context = super().get_serializer_context()
-        context['subject'] = self.subject
-        return context
+    def perform_update(self, serializer):
+        guardian = serializer.save()
+        set_comment(f'Updated guardian <ID-{guardian.id}>')
